﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Trees
{
    internal class BinaryTreeNode<T>
    {
        public T Value { get; set; }
        public BinaryTreeNode<T> Left { get; set; }
        public BinaryTreeNode<T> Right { get; set; }

        public BinaryTreeNode(T value)
        {
            Value = value;
        }
    }

    internal class BinaryTree<T> where T : IComparable<T>
    {
        public BinaryTreeNode<T> Root { get; set; }

        public BinaryTree(T initialValue)
        {
            Root = new BinaryTreeNode<T>(initialValue);
        }

        public void Add(T value)
        {
            BinaryTreeNode<T> current = Root;
            int compared = value.CompareTo(current.Value);

            while (compared != 0)
            {
                if (compared < 0)
                {
                    if (current.Left == null)
                    {
                        current.Left = new BinaryTreeNode<T>(value);
                        return;
                    }
                    else
                    {
                        current = current.Left;
                    }
                }
                else
                {
                    if (current.Right == null)
                    {
                        current.Right = new BinaryTreeNode<T>(value);
                        return;
                    }
                    else
                    {
                        current = current.Right;
                    }
                }

                compared = value.CompareTo(current.Value);
            }

        }

        public bool ContainsElement(T value)
        {
            BinaryTreeNode<T> current = Root;
            int compared = value.CompareTo(current.Value);

            while (compared != 0)
            {
                if (compared < 0)
                {
                    if (current.Left == null)
                    {
                        return false;
                    }
                    else
                    {
                        current = current.Left;
                    }
                }
                else
                {
                    if (current.Right == null)
                    {
                        return false;
                    }
                    else
                    {
                        current = current.Right;
                    }
                }

                compared = value.CompareTo(current.Value);
            }

            return true;
        }

        public List<T> SortedList()
        {
            return DFS(Root, new List<T>());
        }

        private List<T> DFS(BinaryTreeNode<T> current, List<T> list)
        {
            if (current.Left != null)
            {
                DFS(current.Left, list);
            }

            list.Add(current.Value);

            if (current.Right != null)
            {
                DFS(current.Right, list);
            }

            return list;
        }
    }
}
